"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiContext = void 0;
const attachment_registry_1 = require("./attachment-registry");
const uuid = require("uuid-random");
const util_1 = require("./util");
const qs = require("querystring");
const NOOP_STRING = input => (input.toString());
if (global.VALORY_CONTENT_HANDLER == null) {
    global.VALORY_CONTENT_HANDLER = {
        parserMap: {
            "application/json": JSON.parse,
            "application/x-www-form-urlencoded": qs.parse,
        },
        serializerMap: {
            "application/json": JSON.stringify,
            "application/x-www-form-urlencoded": qs.stringify,
            "application/octet-stream": (x) => x
        }
    };
}
class ApiContext {
    constructor(request) {
        this.response = {
            body: {},
            headers: {},
            statusCode: 200
        };
        this.attachments = new attachment_registry_1.AttachmentRegistry();
        const headers = util_1.lowercaseKeys(request.headers);
        const contentType = request.headers["content-type"] || ApiContext.defaultContentType;
        const body = ApiContext.parse(contentType, request.rawBody);
        this.requestId = request.requestId || ApiContext.requestIdGenerator();
        this.request = {
            formData: body,
            body,
            headers,
            path: request.path,
            method: request.method,
            rawBody: request.rawBody,
            pathParams: request.pathParams || {},
            queryParams: ApiContext.parse("application/x-www-form-urlencoded", request.query) || {}
        };
    }
    static registerParser(type, parser) {
        ApiContext.parserMap[type] = parser;
    }
    static registerSerializer(type, serializer) {
        ApiContext.serializerMap[type] = serializer;
    }
    static parse(contentType = ApiContext.defaultContentType, input) {
        if (input === "" || input == null) {
            return undefined;
        }
        try {
            return (ApiContext.parserMap[contentType] || NOOP_STRING)(input);
        }
        catch (e) {
            return input;
        }
    }
    static serialize(contentType = ApiContext.defaultContentType, input) {
        try {
            return (ApiContext.serializerMap[contentType] || NOOP_STRING)(input);
        }
        catch (e) {
            return input.toString();
        }
    }
    prepareHeaders() {
        const headers = this.response.headers;
        headers["content-type"] = this.responseContentType();
        headers["content-length"] = Buffer.byteLength(this.serializeResponse());
        return headers;
    }
    responseContentType() {
        return this.response.headers["content-type"] || ApiContext.defaultContentType;
    }
    serializeResponse() {
        if (this.response.body == null) {
            return "";
        }
        if (this.serializedResponse) {
            return this.serializedResponse;
        }
        const contentType = this.responseContentType();
        this.serializedResponse = ApiContext.serialize(contentType, this.response.body);
        return this.serializedResponse;
    }
}
exports.ApiContext = ApiContext;
ApiContext.defaultContentType = "application/json";
ApiContext.requestIdGenerator = uuid;
ApiContext.parserMap = global.VALORY_CONTENT_HANDLER.parserMap;
ApiContext.serializerMap = global.VALORY_CONTENT_HANDLER.serializerMap;
